unit lua_global_proc;

interface

uses
  Windows,
  CrystalLUA,
  FWDebugerCore,
  lua_script_provider;

var
  luaContinueStatus: DWORD;
  luaCloseDebugProcessOnFree: Boolean;

  procedure luaInitProcess(const Arg: TLuaArg);
  procedure luaHideDebuger;
  function luaRun(const Arg: TLuaArg): TLuaArg;
  procedure luaSetHardwareBreakpoint(const Args: TLuaArgs);
  procedure luaSetFlag(const Args: TLuaArgs);
  procedure luaStopScript;

  function LuaScript: TLua;

implementation

uses
  debuger_utils, SysUtils;

var
  _LuaScript: TLua = nil;

function LuaScript: TLua;
begin
  if _LuaScript = nil then
    _LuaScript := TLua.Create;
  Result := _LuaScript;
end;

procedure luaInitProcess(const Arg: TLuaArg);
begin
  ScriptProvider.InitProcess(Arg.AsBoolean);
end;

procedure luaHideDebuger;
begin
  HideDebuger(ScriptProvider.GetProcess);
end;

function luaRun(const Arg: TLuaArg): TLuaArg;
begin
  Result.AsBoolean := ScriptProvider.Run;
end;

procedure luaSetHardwareBreakpoint(const Args: TLuaArgs);
begin
  ScriptProvider.SetHardwareBreakpoint(
    Pointer(Args[0].AsInteger),
    THWBPSize(Args[1].AsInteger),
    THWBPMode(Args[2].AsInteger),
    THWBPIndex(Args[3].AsInteger));
end;

procedure luaSetFlag(const Args: TLuaArgs);
begin
  ScriptProvider.SetFlag(
    Args[0].AsInteger,
    Args[1].AsBoolean);
end;

procedure luaStopScript;
begin
  ScriptProvider.StopScript;
end;

var
  Info: PLuaRecordInfo;
initialization
  LuaScript.RegVariable('CloseDebugProcessOnFree', luaCloseDebugProcessOnFree, TypeInfo(Boolean));
  LuaScript.RegProc('InitProcess', @luaInitProcess, 1);
  LuaScript.RegProc('HideDebuger', @luaHideDebuger);
  LuaScript.RegProc('Run', @luaRun);
  LuaScript.RegEnum(TypeInfo(THWBPSize));
  LuaScript.RegEnum(TypeInfo(THWBPMode));
  LuaScript.RegProc('SetHardwareBreakpoint', @luaSetHardwareBreakpoint, 4);
  LuaScript.RegConst('EFLAGS_ZF', EFLAGS_ZF);
  LuaScript.RegProc('SetFlag', @luaSetFlag, 2);
  LuaScript.RegProc('StopScript', @luaStopScript);
  LuaScript.RegConst('DBG_EXCEPTION_NOT_HANDLED', DBG_EXCEPTION_NOT_HANDLED);
  LuaScript.RegConst('DBG_CONTINUE', DBG_CONTINUE);
  LuaScript.RegVariable('ContinueStatus', luaContinueStatus, TypeInfo(DWORD));
  Info := LuaScript.RegRecord('TExceptionRecord', Pointer(SizeOf(Windows.TExceptionRecord)));
  with TExceptionRecord(nil^) do
  begin
    Info.RegField('ExceptionAddress', @ExceptionAddress, TypeInfo(DWORD));
  end;

finalization

  FreeAndNil(_LuaScript);

end.
